//-----------------------------------------------------------------------------|
//  - PROOF OF CONCEPT -                                                       |
//                                                                             | 
// Arduino with nRF24L01+ acting as a flash trigger for the                    |
// Elinchrom EL-Skyport system (emulates the EL-Skyport Transmitter)           |
// * Starts up nRF24L01+ with the correct parameter set for the                |
//   Elinchrom Skyport System.                                                 |
// * Sends out a flash trigger message every 500ms                             |
//                                                                             |
// http://infar.be                                                             |
// 2013                                                                        |
// Share/use as you like, but please give credit ;)                            |
//-----------------------------------------------------------------------------|

//------------------------------------------------------------------------------
// Parameters for the Elinchrom Skyport system
//------------------------------------------------------------------------------

// RF Channel:
// 1 = 2456 MHz (0x38)
// 2 = 2458 MHz (0x3A)
// 3 = 2460 MHz (0x3C)
// 4 = 2462 MHz (0x3E)
// 5 = 2469 MHz (0x45)
// 6 = 2471 MHz (0x47)
// 7 = 2473 MHz (0x49)
// 8 = 2475 MHz (0x4B)

// RF power: 0 dBm
// Crystal frequency: 16 MHz
// RF data rate: 250 kbps
// Communication mode: Burst mode
// CRC: CRC enabled, 8 bit CRC
// Preamble generation enabled

// One complete message looks like this: AD CB 8F FF 00
//                                       -------- -----
//                                           |      |
//                                        Address Payload

// TX address is 3 Byte: AD CB 8F

// Trigger messages are:
// Group 1:  C0 00
// Group 2:  D0 00
// Group 3:  E0 00
// Group 4:  F0 00
//     All:  FF 00

// Power increase (+) messages are:
// Group 1:  C0 01
// Group 2:  D0 01
// Group 3:  E0 01
// Group 4:  F0 01
//     All:  FF 01

// Power decrease (-) messages are:
// Group 1:  C0 02
// Group 2:  D0 02
// Group 3:  E0 02
// Group 4:  F0 02
//     All:  FF 02

// Modeling light messages are (last byte alternates between 10 and 20 for every new message):
// Group 1:  C0 10
// Group 2:  D0 10
// Group 3:  E0 10
// Group 4:  F0 10
//     All:  FF 10

// Detailed information about the protocol can be found here:
// http://infar.be/index.php?/archives/987-Elinchrom-EL-Skyport-Transmitter-Protocol-and-configuration.html
// Detailed information about the hardware design of the Elinchrom EL-Skyport Transmitter can be found here:
// http://infar.be/index.php?/archives/982-Elinchrom-EL-Skyport-Transmitter-Opening-design-schematics.html
// Description about this sourcecode and the corresponding schematic is documented here:
// http://infar.be/index.php?/archives/1005-Elinchrom-EL-Skyport-triggered-by-an-Arduino.html

#include <SPI.h>

int  CE_pin = 9;                         // nRF24L01+ CE pin connected to Arduino pin  9
int  CSN_pin = 10;                       // nRF24L01+ CSN pin connected to Arduino pin 10
int  led = 13;                           // LED connected to Arduino pin 13
byte rf_ch = 0x38;                       // Elinchrom channel 1, 2456 MHz

void setup() {

  // Initialize the digital pin for the LED as an output.
  pinMode(led, OUTPUT); 

  //----------------------------------------------------------------------------
  // SPI setup
  //----------------------------------------------------------------------------
  pinMode(CE_pin, OUTPUT);               // Configure Arduino <-> nRF24L01+ CE pin as output
  digitalWrite(CE_pin, LOW);             // Set nRF24L01+ CE pin to LOW
  
  pinMode(CSN_pin, OUTPUT);              // Configure Arduino <-> nRF24L01+ CSN pin as output
  digitalWrite(CSN_pin, HIGH);           // Set nRF24L01+ CSN pin to HIGH
  
  SPI.setClockDivider(SPI_CLOCK_DIV16);  // SPI bus runs at 16MHz / 16 = 1 MHz
  // Start the SPI library
  SPI.begin();
  
  //----------------------------------------------------------------------------
  // Power up and config of the nRF24L01+
  //----------------------------------------------------------------------------

  // Write to register 0x00 (CONFIG)
  digitalWrite(CSN_pin, LOW);            // Set nRF24L01+ CSN pin to LOW
  SPI.transfer(0b00100000);              // Write command for register address 0x00
  SPI.transfer(0b00001010);              // Data Byte: 0000 and EN_CRC = 1, CRCO = 0, PWR_UP = 1, PRIM_RX = 0
  digitalWrite(CSN_pin, HIGH);           // Set nRF24L01+ CSN pin to HIGH
  
  // Wait at least 1.5ms to allow the nRF24L01+ to get from PWR_UP -> standby state
  delay(10);                             // We'll wait 10 ms
  
  // Write to register 0x01 (EN_AA)
  digitalWrite(CSN_pin, LOW);            // Set nRF24L01+ CSN pin to LOW
  SPI.transfer(0b00100001);              // Write command for register address 0x01
  SPI.transfer(0b00000000);              // Data Byte: 00000000 to disable Auto Acknowledgement function
  digitalWrite(CSN_pin, HIGH);           // Set nRF24L01+ CSN pin to HIGH
    
  // Write to register 0x03 (SETUP_AW)
  digitalWrite(CSN_pin, LOW);            // Set nRF24L01+ CSN pin to LOW
  SPI.transfer(0b00100011);              // Write command for register address 0x03
  SPI.transfer(0b00000001);              // Data Byte: 00000001 for 3 Byte address width
  digitalWrite(CSN_pin, HIGH);           // Set nRF24L01+ CSN pin to HIGH
 
  // Write to register 0x04 (SETUP_RETR)
  digitalWrite(CSN_pin, LOW);            // Set nRF24L01+ CSN pin to LOW
  SPI.transfer(0b00100100);              // Write command for register address 0x04
  SPI.transfer(0b00000000);              // Data Byte: 00000000 for disabling retransmit
  digitalWrite(CSN_pin, HIGH);           // Set nRF24L01+ CSN pin to HIGH

  // Write to register 0x05 (RF_CH)
  digitalWrite(CSN_pin, LOW);            // Set nRF24L01+ CSN pin to LOW
  SPI.transfer(0b00100101);              // Write command for register address 0x05
  SPI.transfer(0x38);                    // Data Byte: 56 for 2456 MHz transmit frequency
  digitalWrite(CSN_pin, HIGH);           // Set nRF24L01+ CSN pin to HIGH
  
  // Write to register 0x06 (RF_SETUP)
  digitalWrite(CSN_pin, LOW);            // Set nRF24L01+ CSN pin to LOW
  SPI.transfer(0b00100110);              // Write command for register address 0x06
  SPI.transfer(0b00100110);              // CONT_WAVE = 0, 0, RF_DR_LOW = 1, PLL_LOCK = 0, RF_DR_HIGH = 0, RF_PWR = 11, 0
  digitalWrite(CSN_pin, HIGH);           // Set nRF24L01+ CSN pin to HIGH

  // Write to register 0x10 (TX_ADDR) (Note: LSByte is written first)
  digitalWrite(CSN_pin, LOW);            // Set nRF24L01+ CSN pin to LOW
  SPI.transfer(0b00110000);              // Write command for register address 0x10
  SPI.transfer(0x8F);                    // Write LSByte of address
  SPI.transfer(0xCB);                    // Write second byte of address
  SPI.transfer(0xAD);                    // Write MSByte of address
  digitalWrite(CSN_pin, HIGH);           // Set nRF24L01+ CSN pin to HIGH

}

void loop() {
  
  // W_TX_PAYLOAD command 1010 0000 (1 to 32 LSByte first)
  digitalWrite(CSN_pin, LOW);            // Set nRF24L01+ CSN pin to LOW
  SPI.transfer(0b10100000);              // Command to write to TX_FIFO
  
  // Write payload to TX_FIFO: Trigger command for mode set to "All" on EL-Skyport Transmitter: FF 00 
  SPI.transfer(0xFF);                    // Write first payload byte to TX_FIFO
  SPI.transfer(0x00);                    // Write second payload byte to TX_FIFO
    
  digitalWrite(CSN_pin, HIGH);           // Set nRF24L01+ CSN pin to HIGH
   
  // Send data present in TX FIFO via RF on the specified RF channel
  digitalWrite(CE_pin, LOW);             // Set nRF24L01+ CE pin to LOW (just in case it has not been set LOW before)
  digitalWrite(CE_pin, HIGH);            // Set nRF24L01+ CE pin to HIGH
  delayMicroseconds(50);                 // Wait 50 µs
  digitalWrite(CE_pin, LOW);             // Set nRF24L01+ CE pin to LOW

  delay(500);                            // Wait 500 ms before sending out a new trigger message
 
}




